# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.phone_numbers import csv_download_list_params, csv_download_create_params
from ...types.phone_numbers.csv_download_list_response import CsvDownloadListResponse
from ...types.phone_numbers.csv_download_create_response import CsvDownloadCreateResponse
from ...types.phone_numbers.csv_download_retrieve_response import CsvDownloadRetrieveResponse

__all__ = ["CsvDownloadsResource", "AsyncCsvDownloadsResource"]


class CsvDownloadsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CsvDownloadsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return CsvDownloadsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CsvDownloadsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return CsvDownloadsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        csv_format: Literal["V1", "V2"] | Omit = omit,
        filter: csv_download_create_params.Filter | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CsvDownloadCreateResponse:
        """
        Create a CSV download

        Args:
          csv_format: Which format to use when generating the CSV file. The default for backwards
              compatibility is 'V1'

          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[has_bundle], filter[tag], filter[connection_id], filter[phone_number],
              filter[status], filter[voice.connection_name],
              filter[voice.usage_payment_method], filter[billing_group_id],
              filter[emergency_address_id], filter[customer_reference]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/phone_numbers/csv_downloads",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "csv_format": csv_format,
                        "filter": filter,
                    },
                    csv_download_create_params.CsvDownloadCreateParams,
                ),
            ),
            cast_to=CsvDownloadCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CsvDownloadRetrieveResponse:
        """
        Retrieve a CSV download

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/phone_numbers/csv_downloads/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CsvDownloadRetrieveResponse,
        )

    def list(
        self,
        *,
        page: csv_download_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CsvDownloadListResponse:
        """List CSV downloads

        Args:
          page: Consolidated page parameter (deepObject style).

        Originally: page[size],
              page[number]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/phone_numbers/csv_downloads",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"page": page}, csv_download_list_params.CsvDownloadListParams),
            ),
            cast_to=CsvDownloadListResponse,
        )


class AsyncCsvDownloadsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCsvDownloadsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCsvDownloadsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCsvDownloadsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncCsvDownloadsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        csv_format: Literal["V1", "V2"] | Omit = omit,
        filter: csv_download_create_params.Filter | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CsvDownloadCreateResponse:
        """
        Create a CSV download

        Args:
          csv_format: Which format to use when generating the CSV file. The default for backwards
              compatibility is 'V1'

          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[has_bundle], filter[tag], filter[connection_id], filter[phone_number],
              filter[status], filter[voice.connection_name],
              filter[voice.usage_payment_method], filter[billing_group_id],
              filter[emergency_address_id], filter[customer_reference]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/phone_numbers/csv_downloads",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "csv_format": csv_format,
                        "filter": filter,
                    },
                    csv_download_create_params.CsvDownloadCreateParams,
                ),
            ),
            cast_to=CsvDownloadCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CsvDownloadRetrieveResponse:
        """
        Retrieve a CSV download

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/phone_numbers/csv_downloads/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CsvDownloadRetrieveResponse,
        )

    async def list(
        self,
        *,
        page: csv_download_list_params.Page | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CsvDownloadListResponse:
        """List CSV downloads

        Args:
          page: Consolidated page parameter (deepObject style).

        Originally: page[size],
              page[number]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/phone_numbers/csv_downloads",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"page": page}, csv_download_list_params.CsvDownloadListParams),
            ),
            cast_to=CsvDownloadListResponse,
        )


class CsvDownloadsResourceWithRawResponse:
    def __init__(self, csv_downloads: CsvDownloadsResource) -> None:
        self._csv_downloads = csv_downloads

        self.create = to_raw_response_wrapper(
            csv_downloads.create,
        )
        self.retrieve = to_raw_response_wrapper(
            csv_downloads.retrieve,
        )
        self.list = to_raw_response_wrapper(
            csv_downloads.list,
        )


class AsyncCsvDownloadsResourceWithRawResponse:
    def __init__(self, csv_downloads: AsyncCsvDownloadsResource) -> None:
        self._csv_downloads = csv_downloads

        self.create = async_to_raw_response_wrapper(
            csv_downloads.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            csv_downloads.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            csv_downloads.list,
        )


class CsvDownloadsResourceWithStreamingResponse:
    def __init__(self, csv_downloads: CsvDownloadsResource) -> None:
        self._csv_downloads = csv_downloads

        self.create = to_streamed_response_wrapper(
            csv_downloads.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            csv_downloads.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            csv_downloads.list,
        )


class AsyncCsvDownloadsResourceWithStreamingResponse:
    def __init__(self, csv_downloads: AsyncCsvDownloadsResource) -> None:
        self._csv_downloads = csv_downloads

        self.create = async_to_streamed_response_wrapper(
            csv_downloads.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            csv_downloads.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            csv_downloads.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .rcs_agent import RcsAgent

__all__ = ["RcsAgentResponse"]


class RcsAgentResponse(BaseModel):
    data: Optional[RcsAgent] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["MessageScheduleParams"]


class MessageScheduleParams(TypedDict, total=False):
    to: Required[str]
    """Receiving address (+E.164 formatted phone number or short code)."""

    auto_detect: bool
    """
    Automatically detect if an SMS message is unusually long and exceeds a
    recommended limit of message parts.
    """

    from_: Annotated[str, PropertyInfo(alias="from")]
    """
    Sending address (+E.164 formatted phone number, alphanumeric sender ID, or short
    code).

    **Required if sending with a phone number, short code, or alphanumeric sender
    ID.**
    """

    media_urls: SequenceNotStr[str]
    """A list of media URLs. The total media size must be less than 1 MB.

    **Required for MMS**
    """

    messaging_profile_id: str
    """Unique identifier for a messaging profile.

    **Required if sending via number pool or with an alphanumeric sender ID.**
    """

    send_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """
    ISO 8601 formatted date indicating when to send the message - accurate up till a
    minute.
    """

    subject: str
    """Subject of multimedia message"""

    text: str
    """Message body (i.e., content) as a non-empty string.

    **Required for SMS**
    """

    type: Literal["SMS", "MMS"]
    """The protocol for sending the message, either SMS or MMS."""

    use_profile_webhooks: bool
    """
    If the profile this number is associated with has webhooks, use them for
    delivery notifications. If webhooks are also specified on the message itself,
    they will be attempted first, then those on the profile.
    """

    webhook_failover_url: str
    """
    The failover URL where webhooks related to this message will be sent if sending
    to the primary URL fails.
    """

    webhook_url: str
    """The URL where webhooks related to this message will be sent."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal

import httpx

from .rcs import (
    RcsResource,
    AsyncRcsResource,
    RcsResourceWithRawResponse,
    AsyncRcsResourceWithRawResponse,
    RcsResourceWithStreamingResponse,
    AsyncRcsResourceWithStreamingResponse,
)
from ...types import (
    message_send_params,
    message_schedule_params,
    message_send_group_mms_params,
    message_send_long_code_params,
    message_send_short_code_params,
    message_send_number_pool_params,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.message_send_response import MessageSendResponse
from ...types.message_retrieve_response import MessageRetrieveResponse
from ...types.message_schedule_response import MessageScheduleResponse
from ...types.message_send_group_mms_response import MessageSendGroupMmsResponse
from ...types.message_send_long_code_response import MessageSendLongCodeResponse
from ...types.message_send_short_code_response import MessageSendShortCodeResponse
from ...types.message_cancel_scheduled_response import MessageCancelScheduledResponse
from ...types.message_send_number_pool_response import MessageSendNumberPoolResponse

__all__ = ["MessagesResource", "AsyncMessagesResource"]


class MessagesResource(SyncAPIResource):
    @cached_property
    def rcs(self) -> RcsResource:
        return RcsResource(self._client)

    @cached_property
    def with_raw_response(self) -> MessagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return MessagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MessagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return MessagesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessageRetrieveResponse:
        """
        Note: This API endpoint can only retrieve messages that are no older than 10
        days since their creation. If you require messages older than this, please
        generate an
        [MDR report.](https://developers.telnyx.com/api/v1/mission-control/add-mdr-request)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/messages/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageRetrieveResponse,
        )

    def cancel_scheduled(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessageCancelScheduledResponse:
        """Cancel a scheduled message that has not yet been sent.

        Only messages with
        `status=scheduled` and `send_at` more than a minute from now can be cancelled.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/messages/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageCancelScheduledResponse,
        )

    def schedule(
        self,
        *,
        to: str,
        auto_detect: bool | Omit = omit,
        from_: str | Omit = omit,
        media_urls: SequenceNotStr[str] | Omit = omit,
        messaging_profile_id: str | Omit = omit,
        send_at: Union[str, datetime] | Omit = omit,
        subject: str | Omit = omit,
        text: str | Omit = omit,
        type: Literal["SMS", "MMS"] | Omit = omit,
        use_profile_webhooks: bool | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessageScheduleResponse:
        """
        Schedule a message with a Phone Number, Alphanumeric Sender ID, Short Code or
        Number Pool.

        This endpoint allows you to schedule a message with any messaging resource.
        Current messaging resources include: long-code, short-code, number-pool, and
        alphanumeric-sender-id.

        Args:
          to: Receiving address (+E.164 formatted phone number or short code).

          auto_detect: Automatically detect if an SMS message is unusually long and exceeds a
              recommended limit of message parts.

          from_: Sending address (+E.164 formatted phone number, alphanumeric sender ID, or short
              code).

              **Required if sending with a phone number, short code, or alphanumeric sender
              ID.**

          media_urls: A list of media URLs. The total media size must be less than 1 MB.

              **Required for MMS**

          messaging_profile_id: Unique identifier for a messaging profile.

              **Required if sending via number pool or with an alphanumeric sender ID.**

          send_at: ISO 8601 formatted date indicating when to send the message - accurate up till a
              minute.

          subject: Subject of multimedia message

          text: Message body (i.e., content) as a non-empty string.

              **Required for SMS**

          type: The protocol for sending the message, either SMS or MMS.

          use_profile_webhooks: If the profile this number is associated with has webhooks, use them for
              delivery notifications. If webhooks are also specified on the message itself,
              they will be attempted first, then those on the profile.

          webhook_failover_url: The failover URL where webhooks related to this message will be sent if sending
              to the primary URL fails.

          webhook_url: The URL where webhooks related to this message will be sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/messages/schedule",
            body=maybe_transform(
                {
                    "to": to,
                    "auto_detect": auto_detect,
                    "from_": from_,
                    "media_urls": media_urls,
                    "messaging_profile_id": messaging_profile_id,
                    "send_at": send_at,
                    "subject": subject,
                    "text": text,
                    "type": type,
                    "use_profile_webhooks": use_profile_webhooks,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                message_schedule_params.MessageScheduleParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageScheduleResponse,
        )

    def send(
        self,
        *,
        to: str,
        auto_detect: bool | Omit = omit,
        from_: str | Omit = omit,
        media_urls: SequenceNotStr[str] | Omit = omit,
        messaging_profile_id: str | Omit = omit,
        send_at: Union[str, datetime, None] | Omit = omit,
        subject: str | Omit = omit,
        text: str | Omit = omit,
        type: Literal["SMS", "MMS"] | Omit = omit,
        use_profile_webhooks: bool | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessageSendResponse:
        """
        Send a message with a Phone Number, Alphanumeric Sender ID, Short Code or Number
        Pool.

        This endpoint allows you to send a message with any messaging resource. Current
        messaging resources include: long-code, short-code, number-pool, and
        alphanumeric-sender-id.

        Args:
          to: Receiving address (+E.164 formatted phone number or short code).

          auto_detect: Automatically detect if an SMS message is unusually long and exceeds a
              recommended limit of message parts.

          from_: Sending address (+E.164 formatted phone number, alphanumeric sender ID, or short
              code).

              **Required if sending with a phone number, short code, or alphanumeric sender
              ID.**

          media_urls: A list of media URLs. The total media size must be less than 1 MB.

              **Required for MMS**

          messaging_profile_id: Unique identifier for a messaging profile.

              **Required if sending via number pool or with an alphanumeric sender ID.**

          send_at: ISO 8601 formatted date indicating when to send the message - accurate up till a
              minute.

          subject: Subject of multimedia message

          text: Message body (i.e., content) as a non-empty string.

              **Required for SMS**

          type: The protocol for sending the message, either SMS or MMS.

          use_profile_webhooks: If the profile this number is associated with has webhooks, use them for
              delivery notifications. If webhooks are also specified on the message itself,
              they will be attempted first, then those on the profile.

          webhook_failover_url: The failover URL where webhooks related to this message will be sent if sending
              to the primary URL fails.

          webhook_url: The URL where webhooks related to this message will be sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/messages",
            body=maybe_transform(
                {
                    "to": to,
                    "auto_detect": auto_detect,
                    "from_": from_,
                    "media_urls": media_urls,
                    "messaging_profile_id": messaging_profile_id,
                    "send_at": send_at,
                    "subject": subject,
                    "text": text,
                    "type": type,
                    "use_profile_webhooks": use_profile_webhooks,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                message_send_params.MessageSendParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageSendResponse,
        )

    def send_group_mms(
        self,
        *,
        from_: str,
        to: SequenceNotStr[str],
        media_urls: SequenceNotStr[str] | Omit = omit,
        subject: str | Omit = omit,
        text: str | Omit = omit,
        use_profile_webhooks: bool | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessageSendGroupMmsResponse:
        """
        Send a group MMS message

        Args:
          from_: Phone number, in +E.164 format, used to send the message.

          to: A list of destinations. No more than 8 destinations are allowed.

          media_urls: A list of media URLs. The total media size must be less than 1 MB.

          subject: Subject of multimedia message

          text: Message body (i.e., content) as a non-empty string.

          use_profile_webhooks: If the profile this number is associated with has webhooks, use them for
              delivery notifications. If webhooks are also specified on the message itself,
              they will be attempted first, then those on the profile.

          webhook_failover_url: The failover URL where webhooks related to this message will be sent if sending
              to the primary URL fails.

          webhook_url: The URL where webhooks related to this message will be sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/messages/group_mms",
            body=maybe_transform(
                {
                    "from_": from_,
                    "to": to,
                    "media_urls": media_urls,
                    "subject": subject,
                    "text": text,
                    "use_profile_webhooks": use_profile_webhooks,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                message_send_group_mms_params.MessageSendGroupMmsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageSendGroupMmsResponse,
        )

    def send_long_code(
        self,
        *,
        from_: str,
        to: str,
        auto_detect: bool | Omit = omit,
        media_urls: SequenceNotStr[str] | Omit = omit,
        subject: str | Omit = omit,
        text: str | Omit = omit,
        type: Literal["SMS", "MMS"] | Omit = omit,
        use_profile_webhooks: bool | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessageSendLongCodeResponse:
        """
        Send a long code message

        Args:
          from_: Phone number, in +E.164 format, used to send the message.

          to: Receiving address (+E.164 formatted phone number or short code).

          auto_detect: Automatically detect if an SMS message is unusually long and exceeds a
              recommended limit of message parts.

          media_urls: A list of media URLs. The total media size must be less than 1 MB.

              **Required for MMS**

          subject: Subject of multimedia message

          text: Message body (i.e., content) as a non-empty string.

              **Required for SMS**

          type: The protocol for sending the message, either SMS or MMS.

          use_profile_webhooks: If the profile this number is associated with has webhooks, use them for
              delivery notifications. If webhooks are also specified on the message itself,
              they will be attempted first, then those on the profile.

          webhook_failover_url: The failover URL where webhooks related to this message will be sent if sending
              to the primary URL fails.

          webhook_url: The URL where webhooks related to this message will be sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/messages/long_code",
            body=maybe_transform(
                {
                    "from_": from_,
                    "to": to,
                    "auto_detect": auto_detect,
                    "media_urls": media_urls,
                    "subject": subject,
                    "text": text,
                    "type": type,
                    "use_profile_webhooks": use_profile_webhooks,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                message_send_long_code_params.MessageSendLongCodeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageSendLongCodeResponse,
        )

    def send_number_pool(
        self,
        *,
        messaging_profile_id: str,
        to: str,
        auto_detect: bool | Omit = omit,
        media_urls: SequenceNotStr[str] | Omit = omit,
        subject: str | Omit = omit,
        text: str | Omit = omit,
        type: Literal["SMS", "MMS"] | Omit = omit,
        use_profile_webhooks: bool | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessageSendNumberPoolResponse:
        """
        Send a message using number pool

        Args:
          messaging_profile_id: Unique identifier for a messaging profile.

          to: Receiving address (+E.164 formatted phone number or short code).

          auto_detect: Automatically detect if an SMS message is unusually long and exceeds a
              recommended limit of message parts.

          media_urls: A list of media URLs. The total media size must be less than 1 MB.

              **Required for MMS**

          subject: Subject of multimedia message

          text: Message body (i.e., content) as a non-empty string.

              **Required for SMS**

          type: The protocol for sending the message, either SMS or MMS.

          use_profile_webhooks: If the profile this number is associated with has webhooks, use them for
              delivery notifications. If webhooks are also specified on the message itself,
              they will be attempted first, then those on the profile.

          webhook_failover_url: The failover URL where webhooks related to this message will be sent if sending
              to the primary URL fails.

          webhook_url: The URL where webhooks related to this message will be sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/messages/number_pool",
            body=maybe_transform(
                {
                    "messaging_profile_id": messaging_profile_id,
                    "to": to,
                    "auto_detect": auto_detect,
                    "media_urls": media_urls,
                    "subject": subject,
                    "text": text,
                    "type": type,
                    "use_profile_webhooks": use_profile_webhooks,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                message_send_number_pool_params.MessageSendNumberPoolParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageSendNumberPoolResponse,
        )

    def send_short_code(
        self,
        *,
        from_: str,
        to: str,
        auto_detect: bool | Omit = omit,
        media_urls: SequenceNotStr[str] | Omit = omit,
        subject: str | Omit = omit,
        text: str | Omit = omit,
        type: Literal["SMS", "MMS"] | Omit = omit,
        use_profile_webhooks: bool | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessageSendShortCodeResponse:
        """
        Send a short code message

        Args:
          from_: Phone number, in +E.164 format, used to send the message.

          to: Receiving address (+E.164 formatted phone number or short code).

          auto_detect: Automatically detect if an SMS message is unusually long and exceeds a
              recommended limit of message parts.

          media_urls: A list of media URLs. The total media size must be less than 1 MB.

              **Required for MMS**

          subject: Subject of multimedia message

          text: Message body (i.e., content) as a non-empty string.

              **Required for SMS**

          type: The protocol for sending the message, either SMS or MMS.

          use_profile_webhooks: If the profile this number is associated with has webhooks, use them for
              delivery notifications. If webhooks are also specified on the message itself,
              they will be attempted first, then those on the profile.

          webhook_failover_url: The failover URL where webhooks related to this message will be sent if sending
              to the primary URL fails.

          webhook_url: The URL where webhooks related to this message will be sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/messages/short_code",
            body=maybe_transform(
                {
                    "from_": from_,
                    "to": to,
                    "auto_detect": auto_detect,
                    "media_urls": media_urls,
                    "subject": subject,
                    "text": text,
                    "type": type,
                    "use_profile_webhooks": use_profile_webhooks,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                message_send_short_code_params.MessageSendShortCodeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageSendShortCodeResponse,
        )


class AsyncMessagesResource(AsyncAPIResource):
    @cached_property
    def rcs(self) -> AsyncRcsResource:
        return AsyncRcsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncMessagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMessagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMessagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncMessagesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessageRetrieveResponse:
        """
        Note: This API endpoint can only retrieve messages that are no older than 10
        days since their creation. If you require messages older than this, please
        generate an
        [MDR report.](https://developers.telnyx.com/api/v1/mission-control/add-mdr-request)

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/messages/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageRetrieveResponse,
        )

    async def cancel_scheduled(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessageCancelScheduledResponse:
        """Cancel a scheduled message that has not yet been sent.

        Only messages with
        `status=scheduled` and `send_at` more than a minute from now can be cancelled.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/messages/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageCancelScheduledResponse,
        )

    async def schedule(
        self,
        *,
        to: str,
        auto_detect: bool | Omit = omit,
        from_: str | Omit = omit,
        media_urls: SequenceNotStr[str] | Omit = omit,
        messaging_profile_id: str | Omit = omit,
        send_at: Union[str, datetime] | Omit = omit,
        subject: str | Omit = omit,
        text: str | Omit = omit,
        type: Literal["SMS", "MMS"] | Omit = omit,
        use_profile_webhooks: bool | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessageScheduleResponse:
        """
        Schedule a message with a Phone Number, Alphanumeric Sender ID, Short Code or
        Number Pool.

        This endpoint allows you to schedule a message with any messaging resource.
        Current messaging resources include: long-code, short-code, number-pool, and
        alphanumeric-sender-id.

        Args:
          to: Receiving address (+E.164 formatted phone number or short code).

          auto_detect: Automatically detect if an SMS message is unusually long and exceeds a
              recommended limit of message parts.

          from_: Sending address (+E.164 formatted phone number, alphanumeric sender ID, or short
              code).

              **Required if sending with a phone number, short code, or alphanumeric sender
              ID.**

          media_urls: A list of media URLs. The total media size must be less than 1 MB.

              **Required for MMS**

          messaging_profile_id: Unique identifier for a messaging profile.

              **Required if sending via number pool or with an alphanumeric sender ID.**

          send_at: ISO 8601 formatted date indicating when to send the message - accurate up till a
              minute.

          subject: Subject of multimedia message

          text: Message body (i.e., content) as a non-empty string.

              **Required for SMS**

          type: The protocol for sending the message, either SMS or MMS.

          use_profile_webhooks: If the profile this number is associated with has webhooks, use them for
              delivery notifications. If webhooks are also specified on the message itself,
              they will be attempted first, then those on the profile.

          webhook_failover_url: The failover URL where webhooks related to this message will be sent if sending
              to the primary URL fails.

          webhook_url: The URL where webhooks related to this message will be sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/messages/schedule",
            body=await async_maybe_transform(
                {
                    "to": to,
                    "auto_detect": auto_detect,
                    "from_": from_,
                    "media_urls": media_urls,
                    "messaging_profile_id": messaging_profile_id,
                    "send_at": send_at,
                    "subject": subject,
                    "text": text,
                    "type": type,
                    "use_profile_webhooks": use_profile_webhooks,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                message_schedule_params.MessageScheduleParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageScheduleResponse,
        )

    async def send(
        self,
        *,
        to: str,
        auto_detect: bool | Omit = omit,
        from_: str | Omit = omit,
        media_urls: SequenceNotStr[str] | Omit = omit,
        messaging_profile_id: str | Omit = omit,
        send_at: Union[str, datetime, None] | Omit = omit,
        subject: str | Omit = omit,
        text: str | Omit = omit,
        type: Literal["SMS", "MMS"] | Omit = omit,
        use_profile_webhooks: bool | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessageSendResponse:
        """
        Send a message with a Phone Number, Alphanumeric Sender ID, Short Code or Number
        Pool.

        This endpoint allows you to send a message with any messaging resource. Current
        messaging resources include: long-code, short-code, number-pool, and
        alphanumeric-sender-id.

        Args:
          to: Receiving address (+E.164 formatted phone number or short code).

          auto_detect: Automatically detect if an SMS message is unusually long and exceeds a
              recommended limit of message parts.

          from_: Sending address (+E.164 formatted phone number, alphanumeric sender ID, or short
              code).

              **Required if sending with a phone number, short code, or alphanumeric sender
              ID.**

          media_urls: A list of media URLs. The total media size must be less than 1 MB.

              **Required for MMS**

          messaging_profile_id: Unique identifier for a messaging profile.

              **Required if sending via number pool or with an alphanumeric sender ID.**

          send_at: ISO 8601 formatted date indicating when to send the message - accurate up till a
              minute.

          subject: Subject of multimedia message

          text: Message body (i.e., content) as a non-empty string.

              **Required for SMS**

          type: The protocol for sending the message, either SMS or MMS.

          use_profile_webhooks: If the profile this number is associated with has webhooks, use them for
              delivery notifications. If webhooks are also specified on the message itself,
              they will be attempted first, then those on the profile.

          webhook_failover_url: The failover URL where webhooks related to this message will be sent if sending
              to the primary URL fails.

          webhook_url: The URL where webhooks related to this message will be sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/messages",
            body=await async_maybe_transform(
                {
                    "to": to,
                    "auto_detect": auto_detect,
                    "from_": from_,
                    "media_urls": media_urls,
                    "messaging_profile_id": messaging_profile_id,
                    "send_at": send_at,
                    "subject": subject,
                    "text": text,
                    "type": type,
                    "use_profile_webhooks": use_profile_webhooks,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                message_send_params.MessageSendParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageSendResponse,
        )

    async def send_group_mms(
        self,
        *,
        from_: str,
        to: SequenceNotStr[str],
        media_urls: SequenceNotStr[str] | Omit = omit,
        subject: str | Omit = omit,
        text: str | Omit = omit,
        use_profile_webhooks: bool | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessageSendGroupMmsResponse:
        """
        Send a group MMS message

        Args:
          from_: Phone number, in +E.164 format, used to send the message.

          to: A list of destinations. No more than 8 destinations are allowed.

          media_urls: A list of media URLs. The total media size must be less than 1 MB.

          subject: Subject of multimedia message

          text: Message body (i.e., content) as a non-empty string.

          use_profile_webhooks: If the profile this number is associated with has webhooks, use them for
              delivery notifications. If webhooks are also specified on the message itself,
              they will be attempted first, then those on the profile.

          webhook_failover_url: The failover URL where webhooks related to this message will be sent if sending
              to the primary URL fails.

          webhook_url: The URL where webhooks related to this message will be sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/messages/group_mms",
            body=await async_maybe_transform(
                {
                    "from_": from_,
                    "to": to,
                    "media_urls": media_urls,
                    "subject": subject,
                    "text": text,
                    "use_profile_webhooks": use_profile_webhooks,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                message_send_group_mms_params.MessageSendGroupMmsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageSendGroupMmsResponse,
        )

    async def send_long_code(
        self,
        *,
        from_: str,
        to: str,
        auto_detect: bool | Omit = omit,
        media_urls: SequenceNotStr[str] | Omit = omit,
        subject: str | Omit = omit,
        text: str | Omit = omit,
        type: Literal["SMS", "MMS"] | Omit = omit,
        use_profile_webhooks: bool | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessageSendLongCodeResponse:
        """
        Send a long code message

        Args:
          from_: Phone number, in +E.164 format, used to send the message.

          to: Receiving address (+E.164 formatted phone number or short code).

          auto_detect: Automatically detect if an SMS message is unusually long and exceeds a
              recommended limit of message parts.

          media_urls: A list of media URLs. The total media size must be less than 1 MB.

              **Required for MMS**

          subject: Subject of multimedia message

          text: Message body (i.e., content) as a non-empty string.

              **Required for SMS**

          type: The protocol for sending the message, either SMS or MMS.

          use_profile_webhooks: If the profile this number is associated with has webhooks, use them for
              delivery notifications. If webhooks are also specified on the message itself,
              they will be attempted first, then those on the profile.

          webhook_failover_url: The failover URL where webhooks related to this message will be sent if sending
              to the primary URL fails.

          webhook_url: The URL where webhooks related to this message will be sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/messages/long_code",
            body=await async_maybe_transform(
                {
                    "from_": from_,
                    "to": to,
                    "auto_detect": auto_detect,
                    "media_urls": media_urls,
                    "subject": subject,
                    "text": text,
                    "type": type,
                    "use_profile_webhooks": use_profile_webhooks,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                message_send_long_code_params.MessageSendLongCodeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageSendLongCodeResponse,
        )

    async def send_number_pool(
        self,
        *,
        messaging_profile_id: str,
        to: str,
        auto_detect: bool | Omit = omit,
        media_urls: SequenceNotStr[str] | Omit = omit,
        subject: str | Omit = omit,
        text: str | Omit = omit,
        type: Literal["SMS", "MMS"] | Omit = omit,
        use_profile_webhooks: bool | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessageSendNumberPoolResponse:
        """
        Send a message using number pool

        Args:
          messaging_profile_id: Unique identifier for a messaging profile.

          to: Receiving address (+E.164 formatted phone number or short code).

          auto_detect: Automatically detect if an SMS message is unusually long and exceeds a
              recommended limit of message parts.

          media_urls: A list of media URLs. The total media size must be less than 1 MB.

              **Required for MMS**

          subject: Subject of multimedia message

          text: Message body (i.e., content) as a non-empty string.

              **Required for SMS**

          type: The protocol for sending the message, either SMS or MMS.

          use_profile_webhooks: If the profile this number is associated with has webhooks, use them for
              delivery notifications. If webhooks are also specified on the message itself,
              they will be attempted first, then those on the profile.

          webhook_failover_url: The failover URL where webhooks related to this message will be sent if sending
              to the primary URL fails.

          webhook_url: The URL where webhooks related to this message will be sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/messages/number_pool",
            body=await async_maybe_transform(
                {
                    "messaging_profile_id": messaging_profile_id,
                    "to": to,
                    "auto_detect": auto_detect,
                    "media_urls": media_urls,
                    "subject": subject,
                    "text": text,
                    "type": type,
                    "use_profile_webhooks": use_profile_webhooks,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                message_send_number_pool_params.MessageSendNumberPoolParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageSendNumberPoolResponse,
        )

    async def send_short_code(
        self,
        *,
        from_: str,
        to: str,
        auto_detect: bool | Omit = omit,
        media_urls: SequenceNotStr[str] | Omit = omit,
        subject: str | Omit = omit,
        text: str | Omit = omit,
        type: Literal["SMS", "MMS"] | Omit = omit,
        use_profile_webhooks: bool | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> MessageSendShortCodeResponse:
        """
        Send a short code message

        Args:
          from_: Phone number, in +E.164 format, used to send the message.

          to: Receiving address (+E.164 formatted phone number or short code).

          auto_detect: Automatically detect if an SMS message is unusually long and exceeds a
              recommended limit of message parts.

          media_urls: A list of media URLs. The total media size must be less than 1 MB.

              **Required for MMS**

          subject: Subject of multimedia message

          text: Message body (i.e., content) as a non-empty string.

              **Required for SMS**

          type: The protocol for sending the message, either SMS or MMS.

          use_profile_webhooks: If the profile this number is associated with has webhooks, use them for
              delivery notifications. If webhooks are also specified on the message itself,
              they will be attempted first, then those on the profile.

          webhook_failover_url: The failover URL where webhooks related to this message will be sent if sending
              to the primary URL fails.

          webhook_url: The URL where webhooks related to this message will be sent.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/messages/short_code",
            body=await async_maybe_transform(
                {
                    "from_": from_,
                    "to": to,
                    "auto_detect": auto_detect,
                    "media_urls": media_urls,
                    "subject": subject,
                    "text": text,
                    "type": type,
                    "use_profile_webhooks": use_profile_webhooks,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                },
                message_send_short_code_params.MessageSendShortCodeParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MessageSendShortCodeResponse,
        )


class MessagesResourceWithRawResponse:
    def __init__(self, messages: MessagesResource) -> None:
        self._messages = messages

        self.retrieve = to_raw_response_wrapper(
            messages.retrieve,
        )
        self.cancel_scheduled = to_raw_response_wrapper(
            messages.cancel_scheduled,
        )
        self.schedule = to_raw_response_wrapper(
            messages.schedule,
        )
        self.send = to_raw_response_wrapper(
            messages.send,
        )
        self.send_group_mms = to_raw_response_wrapper(
            messages.send_group_mms,
        )
        self.send_long_code = to_raw_response_wrapper(
            messages.send_long_code,
        )
        self.send_number_pool = to_raw_response_wrapper(
            messages.send_number_pool,
        )
        self.send_short_code = to_raw_response_wrapper(
            messages.send_short_code,
        )

    @cached_property
    def rcs(self) -> RcsResourceWithRawResponse:
        return RcsResourceWithRawResponse(self._messages.rcs)


class AsyncMessagesResourceWithRawResponse:
    def __init__(self, messages: AsyncMessagesResource) -> None:
        self._messages = messages

        self.retrieve = async_to_raw_response_wrapper(
            messages.retrieve,
        )
        self.cancel_scheduled = async_to_raw_response_wrapper(
            messages.cancel_scheduled,
        )
        self.schedule = async_to_raw_response_wrapper(
            messages.schedule,
        )
        self.send = async_to_raw_response_wrapper(
            messages.send,
        )
        self.send_group_mms = async_to_raw_response_wrapper(
            messages.send_group_mms,
        )
        self.send_long_code = async_to_raw_response_wrapper(
            messages.send_long_code,
        )
        self.send_number_pool = async_to_raw_response_wrapper(
            messages.send_number_pool,
        )
        self.send_short_code = async_to_raw_response_wrapper(
            messages.send_short_code,
        )

    @cached_property
    def rcs(self) -> AsyncRcsResourceWithRawResponse:
        return AsyncRcsResourceWithRawResponse(self._messages.rcs)


class MessagesResourceWithStreamingResponse:
    def __init__(self, messages: MessagesResource) -> None:
        self._messages = messages

        self.retrieve = to_streamed_response_wrapper(
            messages.retrieve,
        )
        self.cancel_scheduled = to_streamed_response_wrapper(
            messages.cancel_scheduled,
        )
        self.schedule = to_streamed_response_wrapper(
            messages.schedule,
        )
        self.send = to_streamed_response_wrapper(
            messages.send,
        )
        self.send_group_mms = to_streamed_response_wrapper(
            messages.send_group_mms,
        )
        self.send_long_code = to_streamed_response_wrapper(
            messages.send_long_code,
        )
        self.send_number_pool = to_streamed_response_wrapper(
            messages.send_number_pool,
        )
        self.send_short_code = to_streamed_response_wrapper(
            messages.send_short_code,
        )

    @cached_property
    def rcs(self) -> RcsResourceWithStreamingResponse:
        return RcsResourceWithStreamingResponse(self._messages.rcs)


class AsyncMessagesResourceWithStreamingResponse:
    def __init__(self, messages: AsyncMessagesResource) -> None:
        self._messages = messages

        self.retrieve = async_to_streamed_response_wrapper(
            messages.retrieve,
        )
        self.cancel_scheduled = async_to_streamed_response_wrapper(
            messages.cancel_scheduled,
        )
        self.schedule = async_to_streamed_response_wrapper(
            messages.schedule,
        )
        self.send = async_to_streamed_response_wrapper(
            messages.send,
        )
        self.send_group_mms = async_to_streamed_response_wrapper(
            messages.send_group_mms,
        )
        self.send_long_code = async_to_streamed_response_wrapper(
            messages.send_long_code,
        )
        self.send_number_pool = async_to_streamed_response_wrapper(
            messages.send_number_pool,
        )
        self.send_short_code = async_to_streamed_response_wrapper(
            messages.send_short_code,
        )

    @cached_property
    def rcs(self) -> AsyncRcsResourceWithStreamingResponse:
        return AsyncRcsResourceWithStreamingResponse(self._messages.rcs)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .rc_generate_deeplink_params import RcGenerateDeeplinkParams as RcGenerateDeeplinkParams
from .rc_generate_deeplink_response import RcGenerateDeeplinkResponse as RcGenerateDeeplinkResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...types import (
    Vertical,
    EntityType,
    StockExchange,
    AltBusinessIDType,
    BrandIdentityStatus,
    brand_list_params,
    brand_create_params,
    brand_update_params,
)
from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.vertical import Vertical
from .external_vetting import (
    ExternalVettingResource,
    AsyncExternalVettingResource,
    ExternalVettingResourceWithRawResponse,
    AsyncExternalVettingResourceWithRawResponse,
    ExternalVettingResourceWithStreamingResponse,
    AsyncExternalVettingResourceWithStreamingResponse,
)
from ...types.entity_type import EntityType
from ...types.telnyx_brand import TelnyxBrand
from ...types.stock_exchange import StockExchange
from ...types.brand_list_response import BrandListResponse
from ...types.alt_business_id_type import AltBusinessIDType
from ...types.brand_identity_status import BrandIdentityStatus
from ...types.brand_retrieve_response import BrandRetrieveResponse
from ...types.brand_get_feedback_response import BrandGetFeedbackResponse

__all__ = ["BrandResource", "AsyncBrandResource"]


class BrandResource(SyncAPIResource):
    @cached_property
    def external_vetting(self) -> ExternalVettingResource:
        return ExternalVettingResource(self._client)

    @cached_property
    def with_raw_response(self) -> BrandResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return BrandResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BrandResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return BrandResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        country: str,
        display_name: str,
        email: str,
        entity_type: EntityType,
        vertical: Vertical,
        business_contact_email: str | Omit = omit,
        city: str | Omit = omit,
        company_name: str | Omit = omit,
        ein: str | Omit = omit,
        first_name: str | Omit = omit,
        ip_address: str | Omit = omit,
        is_reseller: bool | Omit = omit,
        last_name: str | Omit = omit,
        mobile_phone: str | Omit = omit,
        mock: bool | Omit = omit,
        phone: str | Omit = omit,
        postal_code: str | Omit = omit,
        state: str | Omit = omit,
        stock_exchange: StockExchange | Omit = omit,
        stock_symbol: str | Omit = omit,
        street: str | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        website: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxBrand:
        """This endpoint is used to create a new brand.

        A brand is an entity created by The
        Campaign Registry (TCR) that represents an organization or a company. It is this
        entity that TCR created campaigns will be associated with. Each brand creation
        will entail an upfront, non-refundable $4 expense.

        Args:
          country: ISO2 2 characters country code. Example: US - United States

          display_name: Display name, marketing name, or DBA name of the brand.

          email: Valid email address of brand support contact.

          entity_type: Entity type behind the brand. This is the form of business establishment.

          vertical: Vertical or industry segment of the brand or campaign.

          business_contact_email: Business contact email.

              Required if `entityType` is `PUBLIC_PROFIT`. Otherwise, it is recommended to
              either omit this field or set it to `null`.

          city: City name

          company_name: (Required for Non-profit/private/public) Legal company name.

          ein: (Required for Non-profit) Government assigned corporate tax ID. EIN is 9-digits
              in U.S.

          first_name: First name of business contact.

          ip_address: IP address of the browser requesting to create brand identity.

          last_name: Last name of business contact.

          mobile_phone: Valid mobile phone number in e.164 international format.

          mock: Mock brand for testing purposes. Defaults to false.

          phone: Valid phone number in e.164 international format.

          postal_code: Postal codes. Use 5 digit zipcode for United States

          state: State. Must be 2 letters code for United States.

          stock_exchange: (Required for public company) stock exchange.

          stock_symbol: (Required for public company) stock symbol.

          street: Street number and name.

          webhook_failover_url: Webhook failover URL for brand status updates.

          webhook_url: Webhook URL for brand status updates.

          website: Brand website URL.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/brand",
            body=maybe_transform(
                {
                    "country": country,
                    "display_name": display_name,
                    "email": email,
                    "entity_type": entity_type,
                    "vertical": vertical,
                    "business_contact_email": business_contact_email,
                    "city": city,
                    "company_name": company_name,
                    "ein": ein,
                    "first_name": first_name,
                    "ip_address": ip_address,
                    "is_reseller": is_reseller,
                    "last_name": last_name,
                    "mobile_phone": mobile_phone,
                    "mock": mock,
                    "phone": phone,
                    "postal_code": postal_code,
                    "state": state,
                    "stock_exchange": stock_exchange,
                    "stock_symbol": stock_symbol,
                    "street": street,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                    "website": website,
                },
                brand_create_params.BrandCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxBrand,
        )

    def retrieve(
        self,
        brand_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrandRetrieveResponse:
        """
        Retrieve a brand by `brandId`.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        return self._get(
            f"/brand/{brand_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BrandRetrieveResponse,
        )

    def update(
        self,
        brand_id: str,
        *,
        country: str,
        display_name: str,
        email: str,
        entity_type: EntityType,
        vertical: Vertical,
        alt_business_id: str | Omit = omit,
        alt_business_id_type: AltBusinessIDType | Omit = omit,
        business_contact_email: str | Omit = omit,
        city: str | Omit = omit,
        company_name: str | Omit = omit,
        ein: str | Omit = omit,
        first_name: str | Omit = omit,
        identity_status: BrandIdentityStatus | Omit = omit,
        ip_address: str | Omit = omit,
        is_reseller: bool | Omit = omit,
        last_name: str | Omit = omit,
        phone: str | Omit = omit,
        postal_code: str | Omit = omit,
        state: str | Omit = omit,
        stock_exchange: StockExchange | Omit = omit,
        stock_symbol: str | Omit = omit,
        street: str | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        website: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxBrand:
        """
        Update a brand's attributes by `brandId`.

        Args:
          country: ISO2 2 characters country code. Example: US - United States

          display_name: Display or marketing name of the brand.

          email: Valid email address of brand support contact.

          entity_type: Entity type behind the brand. This is the form of business establishment.

          vertical: Vertical or industry segment of the brand or campaign.

          alt_business_id: Alternate business identifier such as DUNS, LEI, or GIIN

          alt_business_id_type: An enumeration.

          business_contact_email: Business contact email.

              Required if `entityType` will be changed to `PUBLIC_PROFIT`. Otherwise, it is
              recommended to either omit this field or set it to `null`.

          city: City name

          company_name: (Required for Non-profit/private/public) Legal company name.

          ein: (Required for Non-profit) Government assigned corporate tax ID. EIN is 9-digits
              in U.S.

          first_name: First name of business contact.

          identity_status: The verification status of an active brand

          ip_address: IP address of the browser requesting to create brand identity.

          last_name: Last name of business contact.

          phone: Valid phone number in e.164 international format.

          postal_code: Postal codes. Use 5 digit zipcode for United States

          state: State. Must be 2 letters code for United States.

          stock_exchange: (Required for public company) stock exchange.

          stock_symbol: (Required for public company) stock symbol.

          street: Street number and name.

          webhook_failover_url: Webhook failover URL for brand status updates.

          webhook_url: Webhook URL for brand status updates.

          website: Brand website URL.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        return self._put(
            f"/brand/{brand_id}",
            body=maybe_transform(
                {
                    "country": country,
                    "display_name": display_name,
                    "email": email,
                    "entity_type": entity_type,
                    "vertical": vertical,
                    "alt_business_id": alt_business_id,
                    "alt_business_id_type": alt_business_id_type,
                    "business_contact_email": business_contact_email,
                    "city": city,
                    "company_name": company_name,
                    "ein": ein,
                    "first_name": first_name,
                    "identity_status": identity_status,
                    "ip_address": ip_address,
                    "is_reseller": is_reseller,
                    "last_name": last_name,
                    "phone": phone,
                    "postal_code": postal_code,
                    "state": state,
                    "stock_exchange": stock_exchange,
                    "stock_symbol": stock_symbol,
                    "street": street,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                    "website": website,
                },
                brand_update_params.BrandUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxBrand,
        )

    def list(
        self,
        *,
        brand_id: str | Omit = omit,
        country: str | Omit = omit,
        display_name: str | Omit = omit,
        entity_type: str | Omit = omit,
        page: int | Omit = omit,
        records_per_page: int | Omit = omit,
        sort: Literal[
            "assignedCampaignsCount",
            "-assignedCampaignsCount",
            "brandId",
            "-brandId",
            "createdAt",
            "-createdAt",
            "displayName",
            "-displayName",
            "identityStatus",
            "-identityStatus",
            "status",
            "-status",
            "tcrBrandId",
            "-tcrBrandId",
        ]
        | Omit = omit,
        state: str | Omit = omit,
        tcr_brand_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrandListResponse:
        """
        This endpoint is used to list all brands associated with your organization.

        Args:
          brand_id: Filter results by the Telnyx Brand id

          records_per_page: number of records per page. maximum of 500

          sort: Specifies the sort order for results. If not given, results are sorted by
              createdAt in descending order.

          tcr_brand_id: Filter results by the TCR Brand id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/brand",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "brand_id": brand_id,
                        "country": country,
                        "display_name": display_name,
                        "entity_type": entity_type,
                        "page": page,
                        "records_per_page": records_per_page,
                        "sort": sort,
                        "state": state,
                        "tcr_brand_id": tcr_brand_id,
                    },
                    brand_list_params.BrandListParams,
                ),
            ),
            cast_to=BrandListResponse,
        )

    def delete(
        self,
        brand_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """Delete Brand.

        This endpoint is used to delete a brand. Note the brand cannot be
        deleted if it contains one or more active campaigns, the campaigns need to be
        inactive and at least 3 months old due to billing purposes.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        return self._delete(
            f"/brand/{brand_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def get_feedback(
        self,
        brand_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrandGetFeedbackResponse:
        """Get feedback about a brand by ID.

        This endpoint can be used after creating or
        revetting a brand.

        Possible values for `.category[].id`:

        - `TAX_ID` - Data mismatch related to tax id and its associated properties.
        - `STOCK_SYMBOL` - Non public entity registered as a public for profit entity or
          the stock information mismatch.
        - `GOVERNMENT_ENTITY` - Non government entity registered as a government entity.
          Must be a U.S. government entity.
        - `NONPROFIT` - Not a recognized non-profit entity. No IRS tax-exempt status
          found.
        - `OTHERS` - Details of the data misrepresentation if any.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        return self._get(
            f"/brand/feedback/{brand_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BrandGetFeedbackResponse,
        )

    def resend_2fa_email(
        self,
        brand_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Resend brand 2FA email

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            f"/brand/{brand_id}/2faEmail",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def revet(
        self,
        brand_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """This operation allows you to revet the brand.

        However, revetting is allowed once
        after the successful brand registration and thereafter limited to once every 3
        months.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        return self._put(
            f"/brand/{brand_id}/revet",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncBrandResource(AsyncAPIResource):
    @cached_property
    def external_vetting(self) -> AsyncExternalVettingResource:
        return AsyncExternalVettingResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncBrandResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBrandResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBrandResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncBrandResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        country: str,
        display_name: str,
        email: str,
        entity_type: EntityType,
        vertical: Vertical,
        business_contact_email: str | Omit = omit,
        city: str | Omit = omit,
        company_name: str | Omit = omit,
        ein: str | Omit = omit,
        first_name: str | Omit = omit,
        ip_address: str | Omit = omit,
        is_reseller: bool | Omit = omit,
        last_name: str | Omit = omit,
        mobile_phone: str | Omit = omit,
        mock: bool | Omit = omit,
        phone: str | Omit = omit,
        postal_code: str | Omit = omit,
        state: str | Omit = omit,
        stock_exchange: StockExchange | Omit = omit,
        stock_symbol: str | Omit = omit,
        street: str | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        website: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxBrand:
        """This endpoint is used to create a new brand.

        A brand is an entity created by The
        Campaign Registry (TCR) that represents an organization or a company. It is this
        entity that TCR created campaigns will be associated with. Each brand creation
        will entail an upfront, non-refundable $4 expense.

        Args:
          country: ISO2 2 characters country code. Example: US - United States

          display_name: Display name, marketing name, or DBA name of the brand.

          email: Valid email address of brand support contact.

          entity_type: Entity type behind the brand. This is the form of business establishment.

          vertical: Vertical or industry segment of the brand or campaign.

          business_contact_email: Business contact email.

              Required if `entityType` is `PUBLIC_PROFIT`. Otherwise, it is recommended to
              either omit this field or set it to `null`.

          city: City name

          company_name: (Required for Non-profit/private/public) Legal company name.

          ein: (Required for Non-profit) Government assigned corporate tax ID. EIN is 9-digits
              in U.S.

          first_name: First name of business contact.

          ip_address: IP address of the browser requesting to create brand identity.

          last_name: Last name of business contact.

          mobile_phone: Valid mobile phone number in e.164 international format.

          mock: Mock brand for testing purposes. Defaults to false.

          phone: Valid phone number in e.164 international format.

          postal_code: Postal codes. Use 5 digit zipcode for United States

          state: State. Must be 2 letters code for United States.

          stock_exchange: (Required for public company) stock exchange.

          stock_symbol: (Required for public company) stock symbol.

          street: Street number and name.

          webhook_failover_url: Webhook failover URL for brand status updates.

          webhook_url: Webhook URL for brand status updates.

          website: Brand website URL.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/brand",
            body=await async_maybe_transform(
                {
                    "country": country,
                    "display_name": display_name,
                    "email": email,
                    "entity_type": entity_type,
                    "vertical": vertical,
                    "business_contact_email": business_contact_email,
                    "city": city,
                    "company_name": company_name,
                    "ein": ein,
                    "first_name": first_name,
                    "ip_address": ip_address,
                    "is_reseller": is_reseller,
                    "last_name": last_name,
                    "mobile_phone": mobile_phone,
                    "mock": mock,
                    "phone": phone,
                    "postal_code": postal_code,
                    "state": state,
                    "stock_exchange": stock_exchange,
                    "stock_symbol": stock_symbol,
                    "street": street,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                    "website": website,
                },
                brand_create_params.BrandCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxBrand,
        )

    async def retrieve(
        self,
        brand_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrandRetrieveResponse:
        """
        Retrieve a brand by `brandId`.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        return await self._get(
            f"/brand/{brand_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BrandRetrieveResponse,
        )

    async def update(
        self,
        brand_id: str,
        *,
        country: str,
        display_name: str,
        email: str,
        entity_type: EntityType,
        vertical: Vertical,
        alt_business_id: str | Omit = omit,
        alt_business_id_type: AltBusinessIDType | Omit = omit,
        business_contact_email: str | Omit = omit,
        city: str | Omit = omit,
        company_name: str | Omit = omit,
        ein: str | Omit = omit,
        first_name: str | Omit = omit,
        identity_status: BrandIdentityStatus | Omit = omit,
        ip_address: str | Omit = omit,
        is_reseller: bool | Omit = omit,
        last_name: str | Omit = omit,
        phone: str | Omit = omit,
        postal_code: str | Omit = omit,
        state: str | Omit = omit,
        stock_exchange: StockExchange | Omit = omit,
        stock_symbol: str | Omit = omit,
        street: str | Omit = omit,
        webhook_failover_url: str | Omit = omit,
        webhook_url: str | Omit = omit,
        website: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> TelnyxBrand:
        """
        Update a brand's attributes by `brandId`.

        Args:
          country: ISO2 2 characters country code. Example: US - United States

          display_name: Display or marketing name of the brand.

          email: Valid email address of brand support contact.

          entity_type: Entity type behind the brand. This is the form of business establishment.

          vertical: Vertical or industry segment of the brand or campaign.

          alt_business_id: Alternate business identifier such as DUNS, LEI, or GIIN

          alt_business_id_type: An enumeration.

          business_contact_email: Business contact email.

              Required if `entityType` will be changed to `PUBLIC_PROFIT`. Otherwise, it is
              recommended to either omit this field or set it to `null`.

          city: City name

          company_name: (Required for Non-profit/private/public) Legal company name.

          ein: (Required for Non-profit) Government assigned corporate tax ID. EIN is 9-digits
              in U.S.

          first_name: First name of business contact.

          identity_status: The verification status of an active brand

          ip_address: IP address of the browser requesting to create brand identity.

          last_name: Last name of business contact.

          phone: Valid phone number in e.164 international format.

          postal_code: Postal codes. Use 5 digit zipcode for United States

          state: State. Must be 2 letters code for United States.

          stock_exchange: (Required for public company) stock exchange.

          stock_symbol: (Required for public company) stock symbol.

          street: Street number and name.

          webhook_failover_url: Webhook failover URL for brand status updates.

          webhook_url: Webhook URL for brand status updates.

          website: Brand website URL.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        return await self._put(
            f"/brand/{brand_id}",
            body=await async_maybe_transform(
                {
                    "country": country,
                    "display_name": display_name,
                    "email": email,
                    "entity_type": entity_type,
                    "vertical": vertical,
                    "alt_business_id": alt_business_id,
                    "alt_business_id_type": alt_business_id_type,
                    "business_contact_email": business_contact_email,
                    "city": city,
                    "company_name": company_name,
                    "ein": ein,
                    "first_name": first_name,
                    "identity_status": identity_status,
                    "ip_address": ip_address,
                    "is_reseller": is_reseller,
                    "last_name": last_name,
                    "phone": phone,
                    "postal_code": postal_code,
                    "state": state,
                    "stock_exchange": stock_exchange,
                    "stock_symbol": stock_symbol,
                    "street": street,
                    "webhook_failover_url": webhook_failover_url,
                    "webhook_url": webhook_url,
                    "website": website,
                },
                brand_update_params.BrandUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=TelnyxBrand,
        )

    async def list(
        self,
        *,
        brand_id: str | Omit = omit,
        country: str | Omit = omit,
        display_name: str | Omit = omit,
        entity_type: str | Omit = omit,
        page: int | Omit = omit,
        records_per_page: int | Omit = omit,
        sort: Literal[
            "assignedCampaignsCount",
            "-assignedCampaignsCount",
            "brandId",
            "-brandId",
            "createdAt",
            "-createdAt",
            "displayName",
            "-displayName",
            "identityStatus",
            "-identityStatus",
            "status",
            "-status",
            "tcrBrandId",
            "-tcrBrandId",
        ]
        | Omit = omit,
        state: str | Omit = omit,
        tcr_brand_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrandListResponse:
        """
        This endpoint is used to list all brands associated with your organization.

        Args:
          brand_id: Filter results by the Telnyx Brand id

          records_per_page: number of records per page. maximum of 500

          sort: Specifies the sort order for results. If not given, results are sorted by
              createdAt in descending order.

          tcr_brand_id: Filter results by the TCR Brand id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/brand",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "brand_id": brand_id,
                        "country": country,
                        "display_name": display_name,
                        "entity_type": entity_type,
                        "page": page,
                        "records_per_page": records_per_page,
                        "sort": sort,
                        "state": state,
                        "tcr_brand_id": tcr_brand_id,
                    },
                    brand_list_params.BrandListParams,
                ),
            ),
            cast_to=BrandListResponse,
        )

    async def delete(
        self,
        brand_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """Delete Brand.

        This endpoint is used to delete a brand. Note the brand cannot be
        deleted if it contains one or more active campaigns, the campaigns need to be
        inactive and at least 3 months old due to billing purposes.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        return await self._delete(
            f"/brand/{brand_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def get_feedback(
        self,
        brand_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BrandGetFeedbackResponse:
        """Get feedback about a brand by ID.

        This endpoint can be used after creating or
        revetting a brand.

        Possible values for `.category[].id`:

        - `TAX_ID` - Data mismatch related to tax id and its associated properties.
        - `STOCK_SYMBOL` - Non public entity registered as a public for profit entity or
          the stock information mismatch.
        - `GOVERNMENT_ENTITY` - Non government entity registered as a government entity.
          Must be a U.S. government entity.
        - `NONPROFIT` - Not a recognized non-profit entity. No IRS tax-exempt status
          found.
        - `OTHERS` - Details of the data misrepresentation if any.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        return await self._get(
            f"/brand/feedback/{brand_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BrandGetFeedbackResponse,
        )

    async def resend_2fa_email(
        self,
        brand_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Resend brand 2FA email

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            f"/brand/{brand_id}/2faEmail",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def revet(
        self,
        brand_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """This operation allows you to revet the brand.

        However, revetting is allowed once
        after the successful brand registration and thereafter limited to once every 3
        months.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not brand_id:
            raise ValueError(f"Expected a non-empty value for `brand_id` but received {brand_id!r}")
        return await self._put(
            f"/brand/{brand_id}/revet",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class BrandResourceWithRawResponse:
    def __init__(self, brand: BrandResource) -> None:
        self._brand = brand

        self.create = to_raw_response_wrapper(
            brand.create,
        )
        self.retrieve = to_raw_response_wrapper(
            brand.retrieve,
        )
        self.update = to_raw_response_wrapper(
            brand.update,
        )
        self.list = to_raw_response_wrapper(
            brand.list,
        )
        self.delete = to_raw_response_wrapper(
            brand.delete,
        )
        self.get_feedback = to_raw_response_wrapper(
            brand.get_feedback,
        )
        self.resend_2fa_email = to_raw_response_wrapper(
            brand.resend_2fa_email,
        )
        self.revet = to_raw_response_wrapper(
            brand.revet,
        )

    @cached_property
    def external_vetting(self) -> ExternalVettingResourceWithRawResponse:
        return ExternalVettingResourceWithRawResponse(self._brand.external_vetting)


class AsyncBrandResourceWithRawResponse:
    def __init__(self, brand: AsyncBrandResource) -> None:
        self._brand = brand

        self.create = async_to_raw_response_wrapper(
            brand.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            brand.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            brand.update,
        )
        self.list = async_to_raw_response_wrapper(
            brand.list,
        )
        self.delete = async_to_raw_response_wrapper(
            brand.delete,
        )
        self.get_feedback = async_to_raw_response_wrapper(
            brand.get_feedback,
        )
        self.resend_2fa_email = async_to_raw_response_wrapper(
            brand.resend_2fa_email,
        )
        self.revet = async_to_raw_response_wrapper(
            brand.revet,
        )

    @cached_property
    def external_vetting(self) -> AsyncExternalVettingResourceWithRawResponse:
        return AsyncExternalVettingResourceWithRawResponse(self._brand.external_vetting)


class BrandResourceWithStreamingResponse:
    def __init__(self, brand: BrandResource) -> None:
        self._brand = brand

        self.create = to_streamed_response_wrapper(
            brand.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            brand.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            brand.update,
        )
        self.list = to_streamed_response_wrapper(
            brand.list,
        )
        self.delete = to_streamed_response_wrapper(
            brand.delete,
        )
        self.get_feedback = to_streamed_response_wrapper(
            brand.get_feedback,
        )
        self.resend_2fa_email = to_streamed_response_wrapper(
            brand.resend_2fa_email,
        )
        self.revet = to_streamed_response_wrapper(
            brand.revet,
        )

    @cached_property
    def external_vetting(self) -> ExternalVettingResourceWithStreamingResponse:
        return ExternalVettingResourceWithStreamingResponse(self._brand.external_vetting)


class AsyncBrandResourceWithStreamingResponse:
    def __init__(self, brand: AsyncBrandResource) -> None:
        self._brand = brand

        self.create = async_to_streamed_response_wrapper(
            brand.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            brand.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            brand.update,
        )
        self.list = async_to_streamed_response_wrapper(
            brand.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            brand.delete,
        )
        self.get_feedback = async_to_streamed_response_wrapper(
            brand.get_feedback,
        )
        self.resend_2fa_email = async_to_streamed_response_wrapper(
            brand.resend_2fa_email,
        )
        self.revet = async_to_streamed_response_wrapper(
            brand.revet,
        )

    @cached_property
    def external_vetting(self) -> AsyncExternalVettingResourceWithStreamingResponse:
        return AsyncExternalVettingResourceWithStreamingResponse(self._brand.external_vetting)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ...._models import BaseModel
from ...rcs_agent import RcsAgent
from ...pagination_meta import PaginationMeta

__all__ = ["AgentListResponse"]


class AgentListResponse(BaseModel):
    data: Optional[List[RcsAgent]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import user_address_list_params, user_address_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.user_address_list_response import UserAddressListResponse
from ..types.user_address_create_response import UserAddressCreateResponse
from ..types.user_address_retrieve_response import UserAddressRetrieveResponse

__all__ = ["UserAddressesResource", "AsyncUserAddressesResource"]


class UserAddressesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UserAddressesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return UserAddressesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UserAddressesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return UserAddressesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        business_name: str,
        country_code: str,
        first_name: str,
        last_name: str,
        locality: str,
        street_address: str,
        administrative_area: str | Omit = omit,
        borough: str | Omit = omit,
        customer_reference: str | Omit = omit,
        extended_address: str | Omit = omit,
        neighborhood: str | Omit = omit,
        phone_number: str | Omit = omit,
        postal_code: str | Omit = omit,
        skip_address_verification: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserAddressCreateResponse:
        """
        Creates a user address.

        Args:
          business_name: The business name associated with the user address.

          country_code: The two-character (ISO 3166-1 alpha-2) country code of the user address.

          first_name: The first name associated with the user address.

          last_name: The last name associated with the user address.

          locality: The locality of the user address. For US addresses, this corresponds to the city
              of the address.

          street_address: The primary street address information about the user address.

          administrative_area: The locality of the user address. For US addresses, this corresponds to the
              state of the address.

          borough: The borough of the user address. This field is not used for addresses in the US
              but is used for some international addresses.

          customer_reference: A customer reference string for customer look ups.

          extended_address: Additional street address information about the user address such as, but not
              limited to, unit number or apartment number.

          neighborhood: The neighborhood of the user address. This field is not used for addresses in
              the US but is used for some international addresses.

          phone_number: The phone number associated with the user address.

          postal_code: The postal code of the user address.

          skip_address_verification: An optional boolean value specifying if verification of the address should be
              skipped or not. UserAddresses are generally used for shipping addresses, and
              failure to validate your shipping address will likely result in a failure to
              deliver SIM cards or other items ordered from Telnyx. Do not use this parameter
              unless you are sure that the address is correct even though it cannot be
              validated. If this is set to any value other than true, verification of the
              address will be attempted, and the user address will not be allowed if
              verification fails. If verification fails but suggested values are available
              that might make the address correct, they will be present in the response as
              well. If this value is set to true, then the verification will not be attempted.
              Defaults to false (verification will be performed).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/user_addresses",
            body=maybe_transform(
                {
                    "business_name": business_name,
                    "country_code": country_code,
                    "first_name": first_name,
                    "last_name": last_name,
                    "locality": locality,
                    "street_address": street_address,
                    "administrative_area": administrative_area,
                    "borough": borough,
                    "customer_reference": customer_reference,
                    "extended_address": extended_address,
                    "neighborhood": neighborhood,
                    "phone_number": phone_number,
                    "postal_code": postal_code,
                    "skip_address_verification": skip_address_verification,
                },
                user_address_create_params.UserAddressCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserAddressCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserAddressRetrieveResponse:
        """
        Retrieves the details of an existing user address.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/user_addresses/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserAddressRetrieveResponse,
        )

    def list(
        self,
        *,
        filter: user_address_list_params.Filter | Omit = omit,
        page: user_address_list_params.Page | Omit = omit,
        sort: Literal["created_at", "first_name", "last_name", "business_name", "street_address"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserAddressListResponse:
        """
        Returns a list of your user addresses.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[customer_reference][eq], filter[customer_reference][contains],
              filter[street_address][contains]

          page: Consolidated page parameter (deepObject style). Originally: page[size],
              page[number]

          sort: Specifies the sort order for results. By default sorting direction is ascending.
              To have the results sorted in descending order add the <code> -</code>
              prefix.<br/><br/> That is: <ul>

                <li>
                  <code>street_address</code>: sorts the result by the
                  <code>street_address</code> field in ascending order.
                </li>

                <li>
                  <code>-street_address</code>: sorts the result by the
                  <code>street_address</code> field in descending order.
                </li>
              </ul> <br/> If not given, results are sorted by <code>created_at</code> in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/user_addresses",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                        "sort": sort,
                    },
                    user_address_list_params.UserAddressListParams,
                ),
            ),
            cast_to=UserAddressListResponse,
        )


class AsyncUserAddressesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUserAddressesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncUserAddressesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUserAddressesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncUserAddressesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        business_name: str,
        country_code: str,
        first_name: str,
        last_name: str,
        locality: str,
        street_address: str,
        administrative_area: str | Omit = omit,
        borough: str | Omit = omit,
        customer_reference: str | Omit = omit,
        extended_address: str | Omit = omit,
        neighborhood: str | Omit = omit,
        phone_number: str | Omit = omit,
        postal_code: str | Omit = omit,
        skip_address_verification: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserAddressCreateResponse:
        """
        Creates a user address.

        Args:
          business_name: The business name associated with the user address.

          country_code: The two-character (ISO 3166-1 alpha-2) country code of the user address.

          first_name: The first name associated with the user address.

          last_name: The last name associated with the user address.

          locality: The locality of the user address. For US addresses, this corresponds to the city
              of the address.

          street_address: The primary street address information about the user address.

          administrative_area: The locality of the user address. For US addresses, this corresponds to the
              state of the address.

          borough: The borough of the user address. This field is not used for addresses in the US
              but is used for some international addresses.

          customer_reference: A customer reference string for customer look ups.

          extended_address: Additional street address information about the user address such as, but not
              limited to, unit number or apartment number.

          neighborhood: The neighborhood of the user address. This field is not used for addresses in
              the US but is used for some international addresses.

          phone_number: The phone number associated with the user address.

          postal_code: The postal code of the user address.

          skip_address_verification: An optional boolean value specifying if verification of the address should be
              skipped or not. UserAddresses are generally used for shipping addresses, and
              failure to validate your shipping address will likely result in a failure to
              deliver SIM cards or other items ordered from Telnyx. Do not use this parameter
              unless you are sure that the address is correct even though it cannot be
              validated. If this is set to any value other than true, verification of the
              address will be attempted, and the user address will not be allowed if
              verification fails. If verification fails but suggested values are available
              that might make the address correct, they will be present in the response as
              well. If this value is set to true, then the verification will not be attempted.
              Defaults to false (verification will be performed).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/user_addresses",
            body=await async_maybe_transform(
                {
                    "business_name": business_name,
                    "country_code": country_code,
                    "first_name": first_name,
                    "last_name": last_name,
                    "locality": locality,
                    "street_address": street_address,
                    "administrative_area": administrative_area,
                    "borough": borough,
                    "customer_reference": customer_reference,
                    "extended_address": extended_address,
                    "neighborhood": neighborhood,
                    "phone_number": phone_number,
                    "postal_code": postal_code,
                    "skip_address_verification": skip_address_verification,
                },
                user_address_create_params.UserAddressCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserAddressCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserAddressRetrieveResponse:
        """
        Retrieves the details of an existing user address.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/user_addresses/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=UserAddressRetrieveResponse,
        )

    async def list(
        self,
        *,
        filter: user_address_list_params.Filter | Omit = omit,
        page: user_address_list_params.Page | Omit = omit,
        sort: Literal["created_at", "first_name", "last_name", "business_name", "street_address"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserAddressListResponse:
        """
        Returns a list of your user addresses.

        Args:
          filter:
              Consolidated filter parameter (deepObject style). Originally:
              filter[customer_reference][eq], filter[customer_reference][contains],
              filter[street_address][contains]

          page: Consolidated page parameter (deepObject style). Originally: page[size],
              page[number]

          sort: Specifies the sort order for results. By default sorting direction is ascending.
              To have the results sorted in descending order add the <code> -</code>
              prefix.<br/><br/> That is: <ul>

                <li>
                  <code>street_address</code>: sorts the result by the
                  <code>street_address</code> field in ascending order.
                </li>

                <li>
                  <code>-street_address</code>: sorts the result by the
                  <code>street_address</code> field in descending order.
                </li>
              </ul> <br/> If not given, results are sorted by <code>created_at</code> in descending order.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/user_addresses",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "page": page,
                        "sort": sort,
                    },
                    user_address_list_params.UserAddressListParams,
                ),
            ),
            cast_to=UserAddressListResponse,
        )


class UserAddressesResourceWithRawResponse:
    def __init__(self, user_addresses: UserAddressesResource) -> None:
        self._user_addresses = user_addresses

        self.create = to_raw_response_wrapper(
            user_addresses.create,
        )
        self.retrieve = to_raw_response_wrapper(
            user_addresses.retrieve,
        )
        self.list = to_raw_response_wrapper(
            user_addresses.list,
        )


class AsyncUserAddressesResourceWithRawResponse:
    def __init__(self, user_addresses: AsyncUserAddressesResource) -> None:
        self._user_addresses = user_addresses

        self.create = async_to_raw_response_wrapper(
            user_addresses.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            user_addresses.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            user_addresses.list,
        )


class UserAddressesResourceWithStreamingResponse:
    def __init__(self, user_addresses: UserAddressesResource) -> None:
        self._user_addresses = user_addresses

        self.create = to_streamed_response_wrapper(
            user_addresses.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            user_addresses.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            user_addresses.list,
        )


class AsyncUserAddressesResourceWithStreamingResponse:
    def __init__(self, user_addresses: AsyncUserAddressesResource) -> None:
        self._user_addresses = user_addresses

        self.create = async_to_streamed_response_wrapper(
            user_addresses.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            user_addresses.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            user_addresses.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["URLShortenerSettingsParam"]


class URLShortenerSettingsParam(TypedDict, total=False):
    domain: Required[str]
    """One of the domains provided by the Telnyx URL shortener service."""

    prefix: str
    """
    Optional prefix that can be used to identify your brand, and will appear in the
    Telnyx generated URLs after the domain name.
    """

    replace_blacklist_only: bool
    """
    Use the link replacement tool only for links that are specifically blacklisted
    by Telnyx.
    """

    send_webhooks: bool
    """Receive webhooks for when your replaced links are clicked.

    Webhooks are sent to the webhooks on the messaging profile.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["CallReferStartedWebhookEvent", "Data", "DataPayload"]


class DataPayload(BaseModel):
    call_control_id: Optional[str] = None
    """Unique ID for controlling the call."""

    call_leg_id: Optional[str] = None
    """ID that is unique to the call and can be used to correlate webhook events."""

    call_session_id: Optional[str] = None
    """
    ID that is unique to the call session and can be used to correlate webhook
    events. Call session is a group of related call legs that logically belong to
    the same phone call, e.g. an inbound and outbound leg of a transferred call.
    """

    client_state: Optional[str] = None
    """State received from a command."""

    connection_id: Optional[str] = None
    """Call Control App ID (formerly Telnyx connection ID) used in the call."""

    from_: Optional[str] = FieldInfo(alias="from", default=None)
    """Number or SIP URI placing the call."""

    sip_notify_response: Optional[int] = None
    """SIP NOTIFY event status for tracking the REFER attempt."""

    to: Optional[str] = None
    """Destination number or SIP URI of the call."""


class Data(BaseModel):
    id: Optional[str] = None
    """Identifies the type of resource."""

    event_type: Optional[Literal["call.refer.started"]] = None
    """The type of event being delivered."""

    occurred_at: Optional[datetime] = None
    """ISO 8601 datetime of when the event occurred."""

    payload: Optional[DataPayload] = None

    record_type: Optional[Literal["event"]] = None
    """Identifies the type of the resource."""


class CallReferStartedWebhookEvent(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["RegionIn"]


class RegionIn(BaseModel):
    region_code: Optional[str] = None
    """The region the interface should be deployed to."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

from .._types import SequenceNotStr

__all__ = ["VideoRegionParam"]


class VideoRegionParam(TypedDict, total=False):
    height: Optional[int]
    """Height of the video region"""

    max_columns: Optional[int]
    """Maximum number of columns of the region's placement grid.

    By default, the region has as many columns as needed to layout all the specified
    video sources.
    """

    max_rows: Optional[int]
    """Maximum number of rows of the region's placement grid.

    By default, the region has as many rows as needed to layout all the specified
    video sources.
    """

    video_sources: SequenceNotStr[str]
    """Array of video recording ids to be composed in the region.

    Can be "\\**" to specify all video recordings in the session
    """

    width: Optional[int]
    """Width of the video region"""

    x_pos: Optional[int]
    """
    X axis value (in pixels) of the region's upper left corner relative to the upper
    left corner of the whole room composition viewport.
    """

    y_pos: Optional[int]
    """
    Y axis value (in pixels) of the region's upper left corner relative to the upper
    left corner of the whole room composition viewport.
    """

    z_pos: Optional[int]
    """
    Regions with higher z_pos values are stacked on top of regions with lower z_pos
    values
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .pagination_meta import PaginationMeta

__all__ = ["MessagingOptoutListResponse", "Data"]


class Data(BaseModel):
    created_at: Optional[datetime] = None
    """The timestamp when the opt-out was created"""

    from_: Optional[str] = FieldInfo(alias="from", default=None)
    """
    Sending address (+E.164 formatted phone number, alphanumeric sender ID, or short
    code).
    """

    keyword: Optional[str] = None
    """The keyword that triggered the opt-out."""

    messaging_profile_id: Optional[str] = None
    """Unique identifier for a messaging profile."""

    to: Optional[str] = None
    """Receiving address (+E.164 formatted phone number or short code)."""


class MessagingOptoutListResponse(BaseModel):
    data: Optional[List[Data]] = None

    meta: Optional[PaginationMeta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .user_bundle import UserBundle as UserBundle
from .pagination_response import PaginationResponse as PaginationResponse
from .user_bundle_resource import UserBundleResource as UserBundleResource
from .billing_bundle_summary import BillingBundleSummary as BillingBundleSummary
from .user_bundle_list_params import UserBundleListParams as UserBundleListParams
from .user_bundle_create_params import UserBundleCreateParams as UserBundleCreateParams
from .user_bundle_list_response import UserBundleListResponse as UserBundleListResponse
from .billing_bundle_list_params import BillingBundleListParams as BillingBundleListParams
from .user_bundle_create_response import UserBundleCreateResponse as UserBundleCreateResponse
from .billing_bundle_list_response import BillingBundleListResponse as BillingBundleListResponse
from .user_bundle_retrieve_response import UserBundleRetrieveResponse as UserBundleRetrieveResponse
from .user_bundle_list_unused_params import UserBundleListUnusedParams as UserBundleListUnusedParams
from .user_bundle_deactivate_response import UserBundleDeactivateResponse as UserBundleDeactivateResponse
from .billing_bundle_retrieve_response import BillingBundleRetrieveResponse as BillingBundleRetrieveResponse
from .user_bundle_list_unused_response import UserBundleListUnusedResponse as UserBundleListUnusedResponse
from .user_bundle_list_resources_response import UserBundleListResourcesResponse as UserBundleListResourcesResponse
